/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Copyright (C) 2004 
 *
 * Authors: Erik Eliasson <eliasson@it.kth.se>
 *          Johan Bilien <jobi@via.ecp.fr>
*/

#include"aes.h"

//debug:
#ifdef DEBUG_OUTPUT
#include<iostream>
#include"print_hex.h"
#endif

using namespace std;

#ifndef HAVE_OPENSSL_AES_H

// openssl AES functions using Paulo Barreto <paulo.barreto@terra.com.br>
// implementation, to avoid the requirement for version 0.9.7 of libssl
void AES_set_encrypt_key( unsigned char * key, int key_nb_bits, AES_KEY * aes_key ){
	rijndaelKeySetupEnc( aes_key->key, key, key_nb_bits );
}

void AES_encrypt( const unsigned char * input, unsigned char * output, AES_KEY * aes_key ){
	rijndaelEncrypt( aes_key->key, NB_ROUND, input, output );
}
#endif

AES::AES():key(NULL){
}

AES::AES( unsigned char * key, int key_length ){
	this->key = (AES_KEY *) malloc( sizeof( AES_KEY ) );
	AES_set_encrypt_key( key, key_length*8, this->key );
}

AES::~AES(){
	if( key )
		free( key );
}

void AES::encrypt( const unsigned char * input, unsigned char * output ){
	AES_encrypt( input, output, key );
}

void AES::get_ctr_cipher_stream( unsigned char * output, unsigned int length,
				unsigned char * iv ){
	unsigned long ctr; // Should be 128 bits, but we
	                     // assume that we 32 bits is enough ...
	unsigned char * aes_input, * temp;
	
	aes_input = (unsigned char *)malloc( AES_BLOCK_SIZE );
	temp = (unsigned char *)malloc( AES_BLOCK_SIZE );

	memcpy( aes_input, iv, 12 );

	for( ctr = 0; ctr < length/AES_BLOCK_SIZE; ctr++ ){
		//compute the cipher stream
		((unsigned long *)aes_input)[3] = 
			htonl(ctr + ntohl( ((unsigned long *)iv)[3] ));

		AES_encrypt( aes_input, &output[ctr*AES_BLOCK_SIZE], key );
	}

	// Treat the last bytes:
	((unsigned long *)aes_input)[3] = 
			htonl(ctr + ntohl( ((unsigned long *)iv)[3] ));
	AES_encrypt( aes_input, temp, key );
	memcpy( &output[ctr*AES_BLOCK_SIZE], temp, length % AES_BLOCK_SIZE );

	free( temp );
	free( aes_input );
}
	


void AES::ctr_encrypt( const unsigned char * input, unsigned int input_length,
		 unsigned char * output, unsigned char * iv ){
	unsigned char cipher_stream[input_length];

	get_ctr_cipher_stream( cipher_stream, input_length, iv );

	for( unsigned int i = 0; i < input_length; i++ ){
		output[i] = cipher_stream[i] ^ input[i];
	}
	
}

void AES::ctr_encrypt( unsigned char * data, unsigned int data_length,
		       unsigned char * iv ){
	unsigned char cipher_stream[data_length];

	get_ctr_cipher_stream( cipher_stream, data_length, iv );

	for( unsigned int i = 0; i < data_length; i++ ){
		data[i] ^= cipher_stream[i];
	}
}
